import PyLibrary.funciones_comunes as fc
import pandas as pd
import os
import logging
from openpyxl import load_workbook
from openpyxl.styles import Font
from openpyxl.comments import Comment

def importar_traduccion_manual(tipo_PLC, archivo_maestro, archivo_traduccion, target_lang_code):
    if not os.path.exists(archivo_maestro):
        print("El archivo maestro no existe.")
        return

    master_col = target_lang_code

    df_maestro = fc.read_dataframe_with_cleanup_retries(archivo_maestro)
    df_traduccion = fc.read_dataframe_with_cleanup_retries(archivo_traduccion)

    # Configurar el logger    
    logger = fc.configurar_logger()

    # Crear un diccionario para almacenar las celdas modificadas en el maestro
    celdas_modificadas = {}
    # Crear un diccionario para almacenar las celdas con errores en el archivo fuente
    celdas_con_errores = {}

    # Iterar sobre las filas del archivo de traducción para actualizar el maestro
    for index, fila in df_traduccion.iterrows():
        clave = fila[df_maestro.columns[0]]
        if clave in df_maestro[df_maestro.columns[0]].values:
            valor_traducido = fila[target_lang_code]
            valor_original = df_maestro.loc[df_maestro[df_maestro.columns[0]] == clave, master_col].values[0]
            
            if pd.notnull(valor_traducido) and valor_traducido != "" and str(valor_original) != str(valor_traducido):
                # Verificar si la traducción es válida
                okToSave, Error = fc.verificar_celda_traducida(tipo_PLC, clave, valor_traducido)
                if okToSave:
                    df_maestro.loc[df_maestro[df_maestro.columns[0]] == clave, master_col] = valor_traducido
                    logger.info(f'Fila {index}, Columna {master_col}: "{valor_original}" actualizado a "{valor_traducido}"')
                    
                    # Almacenar la posición de la celda modificada en el maestro
                    fila_excel = df_maestro.index[df_maestro[df_maestro.columns[0]] == clave].tolist()[0] + 2  # +2 porque Excel empieza en 1 y tiene encabezados
                    columna_excel = df_maestro.columns.get_loc(master_col) + 1  # +1 porque Excel empieza en 1
                    celdas_modificadas[(fila_excel, columna_excel)] = valor_traducido
                else:
                    logger.error(f'Fila {index}, Columna {master_col}: No actualizado porque: {Error}')
                    # Almacenar la posición de la celda con error en el archivo fuente
                    fila_excel = index + 2  # +2 porque Excel empieza en 1 y tiene encabezados
                    columna_excel = df_traduccion.columns.get_loc(target_lang_code) + 1  # +1 porque Excel empieza en 1
                    celdas_con_errores[(fila_excel, columna_excel)] = Error

    # Guardar el archivo maestro actualizado
    fc.save_dataframe_with_retries(df_maestro, output_path=archivo_maestro)
    
    # Aplicar formato de negrita a las celdas modificadas en el maestro
    aplicar_formato_celdas_maestro(archivo_maestro, celdas_modificadas)
    
    # Marcar errores en rojo en el archivo fuente
    marcar_errores_archivo_fuente(archivo_traduccion, celdas_con_errores)
    
    print(f"Traducciones manuales importadas y archivo maestro actualizado: {archivo_maestro}.")
    print(f"Se han actualizado {len(celdas_modificadas)} celdas en el maestro y se han encontrado {len(celdas_con_errores)} errores en el archivo fuente.")

def aplicar_formato_celdas_maestro(archivo, celdas_modificadas):
    """
    Aplica formato de negrita a las celdas modificadas en el archivo maestro.
    
    :param archivo: Ruta del archivo Excel maestro.
    :param celdas_modificadas: Diccionario con las celdas modificadas {(fila, columna): valor}.
    """
    workbook = load_workbook(archivo)
    sheet = workbook.active
    
    for (fila, columna), valor in celdas_modificadas.items():
        celda = sheet.cell(row=fila, column=columna)
        celda.font = Font(bold=True)
        celda.value = valor

    workbook.save(archivo)
    print(f"Se han formateado {len(celdas_modificadas)} celdas modificadas en negrita en el archivo maestro.")

def marcar_errores_archivo_fuente(archivo, celdas_con_errores):
    """
    Marca en rojo las celdas con errores en el archivo fuente y añade comentarios.
    
    :param archivo: Ruta del archivo Excel fuente.
    :param celdas_con_errores: Diccionario con las celdas con errores {(fila, columna): mensaje_error}.
    """
    workbook = load_workbook(archivo)
    sheet = workbook.active
    
    for (fila, columna), mensaje_error in celdas_con_errores.items():
        celda = sheet.cell(row=fila, column=columna)
        celda.font = Font(color="FF0000")  # Texto en rojo
        comment = Comment(mensaje_error, "Sistema de Traducción")
        celda.comment = comment
    
    workbook.save(archivo)
    print(f"Se han marcado {len(celdas_con_errores)} celdas con errores en rojo en el archivo fuente.")

def run(tipo_PLC, codigo_columna_maestra, seleccion_idioma):
    archivo_maestro = f".\\data\\1_hmi_master_translates_{tipo_PLC}.xlsx"
    archivo_traduccion = f".\\data\\2_master_export2translate_{tipo_PLC}.xlsx"  
    
    if seleccion_idioma not in fc.IDIOMAS:
        print("Selección inválida.")
    else:
        _, target_lang_code = fc.IDIOMAS[seleccion_idioma]
        importar_traduccion_manual(tipo_PLC, archivo_maestro, archivo_traduccion, target_lang_code)

if __name__ == "__main__":   
    tipo_PLC = "siemens"
    codigo_columna_maestra = "it-IT"
    fc.mostrar_idiomas()
    seleccion_idioma = int(input("Introduce el número del idioma de destino: "))
    run(tipo_PLC, codigo_columna_maestra, seleccion_idioma)